﻿-- Daily Global Check - Garrison Follower Missions plugin
-- Jadya EU-Well of Eternity
local addonName, addonTable = ...

local foreach = table.foreach

local pID
local pTable
local pFaction = UnitFactionGroup("player")
local questsdata, questsorder = {},{}
DGC_GarrMLocalizedStrings = {}

local plugintitle = "Follower Missions"
local pluginicon = "Interface\\Icons\\Achievement_General_CityAttacker2"

local function formattime(s)
 return string.format("%.2d:%.2d:%.2d", s/(60*60), s/60%60, s%60)
end

local function isquestcompleted(questID)
 if not questID or not pTable[questID] then return false,"<no data>" end
 
 local g,r = "|cff00FF00","|cffFF0000"
 local sr = "No Missions"
 local c = false
 local minor = 999999999999999
 local now = time()
 local counter, ready = 0,0
 foreach(pTable[questID].timers, function(k,v)
  if v < minor then
   c = now > v
   sr = c and READY or formattime(v - now)
   minor = v
  end
  if now > v then ready = ready + 1 end
  counter = counter + 1
 end)
 if c then
  sr = ready.." / "..counter.." "..READY
 end
 
 sr = (c and g or r)..sr.."|r"
 return false, sr -- forced false to counter the "hide completed quests" option
end


local plugin_data

-- template = {[ZONE],[N AME],[PREF],[SUFF],{{x,y},{x,y}},defaultmapID,[QUESTTYPE],[MAPICON],showfunc
local function GenerateQuestsData()
 if not DailyGlobalCheck then return end
 wipe(questsdata)
 wipe(questsorder)
 
 local function gs(v)
  return v == nil and "" or v
 end

 -- plugin options
 local svar = DailyGlobalCheck_PluginData[plugintitle]
 local realm_headers = false
 if svar and svar.options then
  realm_headers = svar.options[1]
 end
 if realm_headers then
  questsorder[1] = {}
 else
  questsorder[1] = {{OVERVIEW}}
 end
 --
 
 local function foreachinorder(t,f)
  local keys = {}
  for k,_ in pairs(t) do
    keys[#keys+1] = k
  end
  table.sort(keys,function (a,b)
   return string.lower(a) < string.lower(b)
  end)
  for _,k in ipairs(keys) do
   f(k,t[k])
  end
 end

 foreachinorder(pTable, function(k,v)
  if v.show then
   local faction_color = v.faction == "Horde" and "|cffFF2222" or v.faction == "Alliance" and "|cff00AAFF" or ""
   questsdata[k] = {gs(v.realm),faction_color..gs(v.name).."|r","",""}
   if realm_headers then
    local t
    for _,data in pairs(questsorder[1]) do
	 if data[1] == v.realm then
	  t = data
	  break
	 end
	end
	if not t then
	 table.insert(questsorder[1], {v.realm})
	 t = questsorder[1][table.getn(questsorder[1])]
	end
	table.insert(t, k)
   else
 	table.insert(questsorder[1][1], k)
   end
  end
 end)

 DailyGlobalCheck:SetFramesData(plugin_data["Data"], plugin_data["Order"], plugintitle)
end

local function Initialize()
 if not DailyGlobalCheck then return end
 DailyGlobalCheck:LoadPlugin(plugin_data)
end

local function checkvisibility()
 local found
 foreach(pTable[pID].timers, function(k,v)
  found = true
 end)

 if found then return end
 
 -- no timers available
 pTable[pID].show = nil
 GenerateQuestsData()
end

local function removeunusedmissions()
 foreach(pTable[pID].timers, function(mID, t)
   local found
   foreach(C_Garrison.GetInProgressMissions(), function(k,v)
    if mID == v.missionID then found = true end
   end)
   foreach(C_Garrison.GetCompleteMissions(), function(k,v)
    if mID == v.missionID then found = true end
   end)
   if not found then
    pTable[pID].timers[mID] = nil
   end
  end)
  checkvisibility()
end

local last_opt = {}
local function OnShow()
 removeunusedmissions()
 
 local svar = DailyGlobalCheck_PluginData[plugintitle]
 if not svar.options then return end
 
 local found
 for k,v in pairs(svar.options) do
  if v ~= last_opt[k] then
   found = true
  end
  last_opt[k] = v
 end
 if found then
  GenerateQuestsData()
 end
end

 plugin_data = {
 ["Title"] = plugintitle,
 ["Icon"]  = pluginicon,
 ["Data"]  = questsdata,
 ["Order"] = questsorder,
 ["Overrides"] = {["isquestcompleted"] = isquestcompleted},
 ["OnShow"] = OnShow,
 ["Options"] = {{"Checkbox","Split Realms",true,{}}},
 }

local initialized = false
local eventframe = CreateFrame("FRAME")
eventframe:RegisterEvent("ADDON_LOADED")
eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
eventframe:RegisterEvent("GARRISON_MISSION_STARTED")
eventframe:RegisterEvent("GARRISON_MISSION_COMPLETED")
local function eventhandler(self, event, ...)
 if event == "ADDON_LOADED" and ... == addonName then
  local pName = GetUnitName("player")
  local pRealm = GetRealmName()
  pID = pName.."-"..pRealm
  if not DGC_GarrMLocalizedStrings[pID] then DGC_GarrMLocalizedStrings[pID] = {} end
  
  pTable = DGC_GarrMLocalizedStrings
  if not pTable[pID].timers then pTable[pID].timers = {} end
  pTable[pID].name = pName
  pTable[pID].realm = pRealm
  pTable[pID].faction = pFaction
 
  GenerateQuestsData()
  Initialize()
  initialized = true
 elseif event == "GARRISON_MISSION_COMPLETED" then
  local missionID = ...
  local t = time()
  if pTable[pID].timers[missionID] then
   pTable[pID].timers[missionID] = nil
  end
 removeunusedmissions()
 elseif event == "GARRISON_MISSION_STARTED" then
  local missionID = ...
  local _, st, _, _, et = C_Garrison.GetMissionTimes(missionID)
  local t = time() + st + et
  pTable[pID].timers[missionID] = t
  pTable[pID].show = true
  removeunusedmissions()
  GenerateQuestsData()
 elseif event == "PLAYER_ENTERING_WORLD" then
  if not initialized then Initialize() end
  eventframe:UnregisterEvent("PLAYER_ENTERING_WORLD")
 end
end
eventframe:SetScript("OnEvent", eventhandler)